/* 
 * Copyright (c) 2008-2010, FUJITSU LIMITED
 * All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation and/or
 *    other materials provided with the distribution.
 * 
 * 3. Redistributions with modification must carry prominent notices stating that you changed 
 *    the files and the date of any change.
 * 
 * 4. Neither the name of FUJITSU LIMITED nor the names of its contributors may be used
 *    to endorse or promote products derived from this software without specific prior
 *    written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;LOSS OF USE,DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.fujitsu.reffi.client.flex.model.rpc {

	import flash.events.IEventDispatcher;
	import flash.net.URLVariables;
	
	import mx.rpc.AsyncToken;
	import mx.rpc.http.HTTPService;

	/**
	 * <p>[概 要]</p>
	 * HTTPServiceによるサーバアクセスを行う機能モデルクラスです.
	 * 
	 * <p>[詳 細]</p>
	 * アクセス先URL指定、アクセスメソッド指定（デフォルト：POST）、
	 * 返却フォーマット指定（デフォルト：object）、URLパラメータ指定を行い、<br>
	 * HTTPリクエスト送信、レスポンス受信を行います。<br>
	 * 
	 * <p>[備 考]</p>
	 * 
	 * @example リモートサーバからXMLデータを受信する<p/>
	 * 
	 * 受信するデータ
	 * <listing version="3.0">
	  &lt;?xml version="1.0" encoding="UTF-8"?&gt;
	  &lt;employees&gt;
	      &lt;employee&gt;
	 	       &lt;name&gt;yamada taro&lt;/name&gt;
	 	       &lt;age&gt;32&lt;/age&gt;
	      &lt;/employee&gt;
	      &lt;employee&gt;
	          &lt;name&gt;suzuki ichiro&lt;/name&gt;
	          &lt;age&gt;35&lt;/age&gt;
	      &lt;/employee&gt;
	  &lt;/employees&gt;
	 * </listing>
	 * 
	 * HTTPServiceCoreを使用して受信処理を実装したアクション
	 * <listing version="3.0">
	  package demo.client.form.action {
	      import flash.events.Event;
	 
	      import jp.co.fujitsu.reffi.client.flex.action.BaseAction;
	      import jp.co.fujitsu.reffi.client.flex.controller.ParameterMapping;
	      import jp.co.fujitsu.reffi.client.flex.model.BaseModel;
	      import jp.co.fujitsu.reffi.client.flex.model.events.ModelProcessEvent;
	      import jp.co.fujitsu.reffi.client.flex.model.rpc.HTTPServiceCore;
	      import jp.co.fujitsu.reffi.client.flex.model.rpc.RPCCore;
	 	
	      import mx.rpc.events.ResultEvent;
	 
	 	   public class SendAction extends BaseAction {
	 	
	          // ①HTTP通信の為の機能モデルを登録します
	  	       override protected function reserveModels(models:Array):void {
	 	            models.push(Class(HTTPServiceCore));
	          }
	 
	          // ②コントローラによってインスタンス化された機能モデルの設定を行います（サーバロジックは任意）
	          override public function nextModel(index:int, prev:ModelProcessEvent, next:BaseModel):Boolean {
	              switch(index) {
	                  case 0:
	                      HTTPServiceCore(next).url ="webController";
	                      HTTPServiceCore(next).addUrlParameter("model.fqcn", "demo.rpc.model.DummyRestResponseModel");
	                      HTTPServiceCore(next).addUrlParameter("forward.page", "DummyRestResponse.xml");
	                      HTTPServiceCore(next).resultFormat = RPCCore.E4X;
	                      break;
	              }
	              return true;
	          }
	 	
	          // ③受信結果を取得します
	          override public function successForward(index:int, model:BaseModel, resultEvent:Event):void {
	              trace(ResultEvent(resultEvent).result.employee[0].name);
	          }
	      }
	  }
	 * </listing>
	 * 
	 * 実行結果
	 * <listing version="3.0">
	 * yamada taro
	 * </listing>
	 * 
	 * <p>Copyright (c) 2008-2009 FUJITSU Japan All rights reserved.</p>
	 * @author Project Reffi
	 */ 
	public class HTTPServiceCore extends RPCCore {
		
		// アクセス先URL
		private var _url:String;
		
		// アクセスメソッド
		private var _httpMethod:String;
		
		// 受信データフォーマット
		private var _resultFormat:String = RPCCore.OBJECT;
		
		// URLパラメータ
		private var _urlParameter:Object;
		
		
		/**
		 * <p>[概 要]</p>
		 * アクセスを行う対象URLです.
		 * 
		 * <p>[詳 細]</p>
		 * 同コンテキストルートにアクセスする場合、ドメイン指定は不要です。
		 * 
		 * <p>[備 考]</p>
		 * 
		 */ 
		public function get url():String{
			return this._url;
		}
		public function set url(url:String):void{
			this._url = url;
		}
		
		/**
		 * <p>[概 要]</p>
		 * HTTPアクセスメソッドです.
		 * 
		 * <p>[詳 細]</p>
		 * POST、GETを文字列指定します。
		 * 
		 * <p>[備 考]</p>
		 * 
		 * @default "POST"
		 */ 
		public function get httpMethod():String{
			return this._httpMethod;
		}
		public function set httpMethod(httpMethod:String):void{
			this._httpMethod = httpMethod;
		}

		/**
		 * <p>[概 要]</p>
		 * 受信データフォーマットです.
		 * 
		 * <p>[詳 細]</p>
		 * 「object」「e4x」「xml」が指定可能です。
		 * 
		 * <p>[備 考]</p>
		 * 
		 * @default object
		 * @see jp.co.fujitsu.reffi.client.flex.model.rpc.RPCCore
		 */ 
		public function get resultFormat():String{
			return this._resultFormat;
		}
		public function set resultFormat(resultFormat:String):void{
			this._resultFormat = resultFormat;
		}

		/**
		 * <p>[概 要]</p>
		 * リクエストする際のURLパラメータです.
		 * 
		 * <p>[詳 細]</p>
		 * このプロパティに設定されたキー=値のセットが送信時のURLパラメタータとして適用されます。
		 * 
		 * 
		 * <p>[備 考]</p>
		 * 
		 */ 
		public function get urlParameter():Object{
			return this._urlParameter;
		}
		public function set urlParameter(urlParamter:Object):void {
			this._urlParameter = urlParamter;
		}

		/**
		 * <p>[概 要]</p>
		 * URLパラメータにパラメータを追加します.
		 * 
		 * <p>[詳 細]</p>
		 * 引数key = 引数valueの形式でurlParameterプロパティに新規パラメータを追加します。
		 * 
		 * <p>[備 考]</p>
		 * valueが配列の場合は引数key=引数value[0]&引数key=引数value[1]の状態で
		 * パラメータが生成されます。
		 * 
		 * @param key URLパラメータキー
		 * @param value URLパラメータ値
		 */ 
		public function addUrlParameter(key:String, value:Object):Object {
			urlParameter[key] = value;
			
			return urlParameter;
		}

		/**
		 * <p>[概 要]</p>
		 * コンストラクタです.
		 * 
		 * <p>[詳 細]</p>
		 * 以下の初期化を行います。
		 * <ol>
		 *   <li>httpMethod = "POST"</li>
		 *   <li>resultFormat = RPCCore.OBJECT</li>
		 *   <li>urlParameter = new URLVariables()</li>
		 * </ol>
		 * 
		 * <p>[備 考]</p>
		 * 
		 */ 
		public function HTTPServiceCore(){
			this.httpMethod = "POST";
			this.resultFormat = RPCCore.OBJECT;
			this.urlParameter = new URLVariables();
		}
		
		/**
		 * <p>[概 要]</p>
		 * HTTPリクエストを送信します.
		 * 
		 * <p>[詳 細]</p>
		 * urlプロパティ値に対して、httpMethodプロパティ値で、urlParameterプロパティ値を
		 * 引数にしてリクエストを送信します。<br>
		 * 受信した値にアクセスする為の型としてresultFormatプロパティ値が適用されます。
		 * 
		 * <p>[備 考]</p>
		 * 
		 */ 
		override protected final function mainProc() : void {
			super.mainProc();
			
			// HTTP通信オブジェクトを生成 
			var service : HTTPService = this.service as HTTPService;
			
			// アクセスURLを指定するテンプレートメソッドをコール
			service.url = this.url;
			
			// アクセスメソッドを指定するテンプレートメソッドをコール
			service.method = this.httpMethod;
			
			// URLパラメータを設定するテンプレートメソッドをコール
			service.request = this.urlParameter;
			
			// 受信データのフォーマットを指定するテンプレートメソッドをコール
			service.resultFormat = this.resultFormat;
						
			// HTTPリクエスト送信
			service.send();
		}
		
		/**
		 * <p>[概 要]</p>
		 * 具象通信オブジェクトを作成します.
		 * 
		 * <p>[詳 細]</p>
		 * HTTPServiceオブジェクトを生成して返却します。
		 * 
		 * <p>[備 考]</p>
		 *
		 * @return HTTPServiceオブジェクト
		 */ 
		override protected final function createConcreteService() : IEventDispatcher {
			return new HTTPService();
		}
		
	}
}
